library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library machxo2;
use machxo2.all;

ENTITY top IS
  PORT (
    -- FTDI Channel A, used normally as JTAG/MPSSE
	M_TCK         		: IN std_logic;   -- ADBUS0
	M_TDI         		: IN std_logic;   -- ADBUS1
	M_TDO         		: OUT std_logic;  -- ADBUS2
	M_TMS         		: IN std_logic;   -- ADBUS3

	ADBUS4              : OUT std_logic; 
	ADBUS7              : IN std_logic;		-- select JTAG/I2C ? 

	ACBUS4              : IN std_logic; -- use as TRST/SRST
	ACBUS5              : IN std_logic;  --

	BDBUS0              : IN std_logic; -- TX from FTDI
	BDBUS1              : OUT std_logic; -- RX to FTDI	

	-- PS MIO on TE0720 or 3.3V FPGA I/O on TE0710/TE0741 modules
	MIO0                : IN std_logic; --	 	
	MIO9                : INOUT std_logic; --
	
	MIO10               : IN std_logic; --	 	
	MIO11               : IN std_logic; --
	
	MIO12               : IN std_logic; --
	MIO13               : OUT std_logic; --
	
	MIO14               : OUT std_logic; -- UART0.RX << BDBUS0
	MIO15               : IN std_logic; -- UART0.TX >> BDBUS1

	X                   : INOUT std_logic_vector(15 downto 0);
	X17         		: OUT std_logic;
	X16         		: IN std_logic;

	-- Module JTAG
	C_TMS         		: OUT std_logic;
	C_TCK         		: OUT std_logic;
	C_TDI         		: OUT std_logic;
	C_TDO         		: IN std_logic;

	-- Module special pins
	PROGMODE      		: INOUT std_logic;
	RESIN      		    : OUT std_logic;
	NOSEQ      		    : IN std_logic;
	PGOOD     		    : IN std_logic;
	MODE      		    : OUT std_logic;
	EN1      		    : OUT std_logic;

	USB_OC     		    : IN std_logic;
		
	-- Not connected pins in left bank
	-- use to sense VCCIO present?
	NC_L0    		    : IN std_logic;

	PIN_RESET  		    : IN std_logic; -- open PIN, pulldown, always GND

	PHY_LED1  		    : OUT std_logic; -- \ drives a bidir/bicolor LED in the RJ45 connector
	PHY_LED1R  		    : OUT std_logic; -- /

	PHY_LED2  		    : OUT std_logic; -- \ another pair.
	PHY_LED2R  		    : OUT std_logic; -- /

	-- Carrier USER I/O, etc
	S1           		: IN std_logic; -- Push Button (usable as external reset)

	-- DIP switches :
	CM0         		: IN std_logic; -- Mode Select, note powered only when module is inserted and/or VCCIO powered
	CM1         		: IN std_logic; 
    -- No CM2 or CM3
	LED1                : OUT std_logic;
	LED2                : OUT std_logic
);
END ENTITY top;

ARCHITECTURE translated OF top IS

 -- onchip oscilllator
 COMPONENT OSCH
	-- synthesis translate_off
	GENERIC (NOM_FREQ: string := "12.09");
	-- synthesis translate_on
	PORT(	STDBY :IN std_logic;
			OSC :OUT std_logic;
			SEDSTDBY :OUT std_logic);
END COMPONENT OSCH;
attribute NOM_FREQ : string;
attribute NOM_FREQ of OSCinst0 : label is "12.09";
-----------------------------------

signal FTDI_RXD : std_logic; -- (alias) data from FTDI 			
signal FTDI_TXD : std_logic; -- (alias) data from (to??)

signal Clk: std_logic;
signal ext_reset_n : std_logic;
signal por_cnt32: std_logic_vector(31 downto 0);
signal POR_CNT4 : STD_LOGIC_VECTOR(3 downto 0) := "0000";
signal POR_DONE: std_logic;  --	set after a few cycles
signal POR_DELAYED: std_logic; -- set after 2^22 cycles

BEGIN

-- Clock source :
OSCInst0: OSCH
	-- synthesis translate_off
	GENERIC MAP ( NOM_FREQ => "12.09" )
	-- synthesis translate_on
	PORT MAP (STDBY=> '0', OSC=> Clk, SEDSTDBY=> open);

-- Output driving and muxing
	MODE  	<= MIO0; --CM1; -- Select SD Boot Mode... must be HARD WIRED!
	EN1 	<= '1';
	
--	LED1 	<= POR_DELAYED;  Blinky blinky !
--	LED2 	<= por_cnt32(21);

	LED1 	<= '1' when CM0='1' else not FTDI_RXD;
	LED2 	<= '1' when CM0='1' else not FTDI_TXD;
	
	PROGMODE <= CM0; -- CM0 developer Mode, uses Switch to control JTAGSEL pin in B2B Connector
	--PROGMODE <= '0'; -- CM0 developer Mode, uses Switch to control JTAGSEL pin in B2B Connector
	
	RESIN	<= S1; -- Manual Module reset

	-- Pmod MIO bank0 multiplexer
	-- Connect Module MIO14,15 (Zynq PS UART) to FTDI Channel B UART
	FTDI_RXD <= BDBUS0;
	BDBUS1	 <= FTDI_TXD;
	MIO14	 <= FTDI_RXD;
	FTDI_TXD <= MIO15;
	
	X17		 <= MIO12;
	MIO13	 <= X16;	

	-- JTAG Chain multiplexer : just pass through...
    C_TMS <= M_TMS;-- when CR3(0)='0' else 'Z';   -- X15
    C_TCK <= M_TCK;-- when CR3(0)='0' else 'Z';   -- X14
    C_TDI <= M_TDI;-- when CR3(0)='0' else 'Z';   -- X13
    M_TDO <= C_TDO;-- when CR3(0)='0' else X(12); -- X12

-- Reset :
	ext_reset_n <= ACBUS4 or ACBUS5;
	POR_DONE <= '1' when POR_CNT4(3 downto 1)="101" else '0';

	process(Clk,ext_reset_n)
	begin
		if (Clk'event and Clk='1') then
			if (ext_reset_n='0') then
				POR_CNT4<="0000";
			elsif (POR_DONE='0') then
				POR_CNT4<=POR_CNT4+"0001";
			end if;
		end if;
	end process;

	process (POR_DONE, Clk)
	begin
		if POR_DONE = '0' then
			POR_DELAYED <= '0'; -- 
			por_cnt32 <= X"00000000";
		elsif Clk'event and Clk = '1' then
			por_cnt32 <= por_cnt32 + X"00000001";
			if por_cnt32(23)='1' then
				POR_DELAYED <= '1';
			end if;
		end if;
	end process;

END ARCHITECTURE translated;